/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform.builders;

import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.ThesaurusIncludeElement;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.L10nUtils;
import fr.exemole.bdfserver.tools.ficheform.FicheFormUtils;
import java.util.Collection;
import java.util.Collections;
import java.util.Set;
import java.util.function.Predicate;
import net.fichotheque.SubsetKey;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.namespaces.BdfSpace;
import net.fichotheque.namespaces.SelectSpace;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.pointeurs.SubsetItemPointeur;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.selection.MotcleQueryBuilder;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionBuilder;
import net.mapeadores.util.conditions.ConditionsConstants;


/**
 *
 * @author Vincent Calame
 */
public class ListThesaurusIncludeElementBuilder {

    private final static Set<Motcle> EMPTY_SET = Collections.emptySet();
    private final static Predicate<Motcle> ALL_PREDICATE = (Motcle motcle) -> {
        return true;
    };
    private final String name;
    private final Thesaurus thesaurus;
    private String label = "";
    private boolean mandatory = false;
    private String listType;
    private SubsetKey destinationSubsetKey;
    private boolean noneAllowed;
    private Set<Motcle> selectedMotcleSet = EMPTY_SET;
    private Predicate<Motcle> filterPredicate = ALL_PREDICATE;
    private boolean newIndexation;
    private Attribute idalphaStyle = null;
    private Attributes attributes = AttributeUtils.EMPTY_ATTRIBUTES;

    public ListThesaurusIncludeElementBuilder(String name, Thesaurus thesaurus) {
        this.name = name;
        this.thesaurus = thesaurus;
    }

    public ListThesaurusIncludeElementBuilder setLabel(String label) {
        if (label == null) {
            throw new IllegalArgumentException("label is null");
        }
        this.label = label;
        return this;
    }

    public ListThesaurusIncludeElementBuilder setMandatory(boolean mandatory) {
        this.mandatory = mandatory;
        return this;
    }

    public ListThesaurusIncludeElementBuilder setAttributes(Attributes attributes) {
        if (attributes == null) {
            this.attributes = AttributeUtils.EMPTY_ATTRIBUTES;
        } else {
            this.attributes = attributes;
        }
        return this;
    }

    public ListThesaurusIncludeElementBuilder setListType(String listType) {
        this.listType = listType;
        return this;
    }

    public ListThesaurusIncludeElementBuilder setNoneAllowed(boolean noneAllowed) {
        this.noneAllowed = noneAllowed;
        return this;
    }

    public ListThesaurusIncludeElementBuilder setIdalphaStyle(Attribute idalphaStyle) {
        this.idalphaStyle = idalphaStyle;
        return this;
    }

    public ListThesaurusIncludeElementBuilder setDestinationSubsetKey(SubsetKey destinationSubsetKey) {
        this.destinationSubsetKey = destinationSubsetKey;
        return this;
    }

    public ListThesaurusIncludeElementBuilder setSelectedMotcleSet(Set<Motcle> selectedMotcleSet) {
        if (selectedMotcleSet == null) {
            this.selectedMotcleSet = EMPTY_SET;
        } else {
            this.selectedMotcleSet = selectedMotcleSet;
        }
        return this;
    }

    public ListThesaurusIncludeElementBuilder setFilterPredicate(Predicate<Motcle> filterPredicate) {
        if (filterPredicate == null) {
            this.filterPredicate = ALL_PREDICATE;
        } else {
            this.filterPredicate = filterPredicate;
        }
        return this;
    }

    public ListThesaurusIncludeElementBuilder setNewIndexation(boolean newIndexation) {
        this.newIndexation = newIndexation;
        return this;
    }

    public ThesaurusIncludeElement.List toListThesaurusIncludeElement() {
        return new InternalListThesaurusIncludeElement(name, label, mandatory, attributes,
                listType, thesaurus, noneAllowed, destinationSubsetKey, idalphaStyle, selectedMotcleSet, filterPredicate, newIndexation);
    }

    public static ThesaurusIncludeElement.List build(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, SubsetIncludeUi includeUi) {
        ListThesaurusIncludeElementBuilder builder = check(fichePointeur, ficheFormParameters, includeUi);
        if (builder == null) {
            return null;
        } else {
            return builder.toListThesaurusIncludeElement();
        }
    }

    @Nullable
    public static ListThesaurusIncludeElementBuilder check(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, SubsetIncludeUi includeUi) {
        ExtendedIncludeKey extendedIncludeKey = includeUi.getExtendedIncludeKey();
        Thesaurus thesaurus = (Thesaurus) ficheFormParameters.getBdfServer().getFichotheque().getSubset(extendedIncludeKey.getSubsetKey());
        if (thesaurus == null) {
            return null;
        }
        if (!FicheFormUtils.hasAccess(ficheFormParameters, includeUi)) {
            return null;
        }
        SubsetItemPointeur pointeur = FicheFormUtils.checkMasterPointeur(fichePointeur, ficheFormParameters, extendedIncludeKey);
        Collection<Liaison> liaisonList = FicheFormUtils.initMotcleList(pointeur, thesaurus, extendedIncludeKey.getRootIncludeKey());
        InputOption inputOption = includeUi.getInputOption();
        boolean isMandatory = FicheFormUtils.isMandatory(includeUi);
        boolean newIndexation = false;
        Set<Motcle> selectedMotcleSet;
        if (pointeur.isEmpty()) {
            selectedMotcleSet = FicheFormUtils.getDefaultMotcleSet(ficheFormParameters, includeUi, thesaurus);
            newIndexation = true;
        } else {
            selectedMotcleSet = FicheFormUtils.getMotcleSet(liaisonList);
        }
        Predicate<Motcle> filterPredicate = getFilterPredicate(ficheFormParameters, includeUi, thesaurus);
        Attribute idalphaStyle = includeUi.getAttributes().getAttribute(BdfSpace.IDALPHASTYLE_KEY);
        if (idalphaStyle == null) {
            idalphaStyle = thesaurus.getThesaurusMetadata().getAttributes().getAttribute(BdfSpace.IDALPHASTYLE_KEY);
        }
        return init(includeUi.getName(), thesaurus)
                .setMandatory(isMandatory)
                .setAttributes(includeUi.getAttributes())
                .setListType(inputOption.getInputType())
                .setIdalphaStyle(idalphaStyle)
                .setNewIndexation(newIndexation)
                .setNoneAllowed(!isMandatory)
                .setSelectedMotcleSet(selectedMotcleSet)
                .setFilterPredicate(filterPredicate)
                .setLabel(L10nUtils.toLabelString(ficheFormParameters, includeUi));
    }

    public static ListThesaurusIncludeElementBuilder init(String name, Thesaurus thesaurus) {
        return new ListThesaurusIncludeElementBuilder(name, thesaurus);
    }

    private static Predicate<Motcle> getFilterPredicate(FicheFormParameters ficheFormParameters, IncludeUi includeUi, Thesaurus thesaurus) {
        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            return ALL_PREDICATE;
        }
        Attribute idalphaFilter = includeUi.getAttributes().getAttribute(SelectSpace.IDALPHA_KEY);
        if (idalphaFilter == null) {
            return ALL_PREDICATE;
        }
        ConditionBuilder conditionBuilder = ConditionBuilder.init(ConditionsConstants.MODE_OR);
        for (String value : idalphaFilter) {
            conditionBuilder.addTextTest(value);
        }
        Condition condition = conditionBuilder.toCondition();
        MotcleQuery motcleQuery = MotcleQueryBuilder.init()
                .addThesaurus(thesaurus)
                .setContentCondition(condition, MotcleQuery.ONLY_IDALPHA_SCOPE)
                .toMotcleQuery();
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(ficheFormParameters.getBdfServer(), ficheFormParameters.getWorkingLang())
                .setSubsetAccessPredicate(EligibilityUtils.ALL_SUBSET_PREDICATE)
                .toSelectionContext();
        return MotcleSelectorBuilder.init(selectionContext)
                .add(motcleQuery)
                .toMotcleSelector();
    }


    private static class InternalListThesaurusIncludeElement implements ThesaurusIncludeElement.List {

        private final String name;
        private final String label;
        private final boolean mandatory;
        private final Attributes attributes;
        private final String listType;
        private final Thesaurus thesaurus;
        private final SubsetKey destinationSubsetKey;
        private final boolean noneAllowed;
        private final Attribute idalphaStyle;
        private final Set<Motcle> selectedMotcleSet;
        private final Predicate<Motcle> filterPredicate;
        private final boolean newIndexation;


        private InternalListThesaurusIncludeElement(String name, String label, boolean mandatory, Attributes attributes, String listType, Thesaurus thesaurus,
                boolean noneAllowed, SubsetKey destinationSubsetKey, Attribute idalphaStyle, Set<Motcle> selectedMotcleSet, Predicate<Motcle> filterPredicate, boolean newIndexation) {
            this.name = name;
            this.label = label;
            this.mandatory = mandatory;
            this.attributes = attributes;
            this.listType = listType;
            this.thesaurus = thesaurus;
            this.noneAllowed = noneAllowed;
            this.destinationSubsetKey = destinationSubsetKey;
            this.idalphaStyle = idalphaStyle;
            this.selectedMotcleSet = selectedMotcleSet;
            this.filterPredicate = filterPredicate;
            this.newIndexation = newIndexation;
        }

        @Override
        public String getIncludeName() {
            return name;
        }

        @Override
        public String getLabel() {
            return label;
        }

        @Override
        public boolean isMandatory() {
            return mandatory;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public String getListType() {
            return listType;
        }

        @Override
        public Thesaurus getThesaurus() {
            return thesaurus;
        }

        @Override
        public boolean isNoneAllowed() {
            return noneAllowed;
        }

        @Override
        public Attribute getIdalphaStyle() {
            return idalphaStyle;
        }

        @Override
        public SubsetKey getDestinationSubsetKey() {
            return destinationSubsetKey;
        }

        @Override
        public Set<Motcle> getSelectedMotcleSet() {
            return selectedMotcleSet;
        }

        @Override
        public Predicate<Motcle> getFilterPredicate() {
            return filterPredicate;
        }

        @Override
        public boolean isNewIndexation() {
            return newIndexation;
        }

    }

}
