/* BdfServer_DirectoryStorage - Copyright (c) 2013-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory;

import fr.exemole.bdfserver.api.configuration.LangConfiguration;
import fr.exemole.bdfserver.api.groups.GroupDef;
import fr.exemole.bdfserver.api.policies.UserAllow;
import fr.exemole.bdfserver.api.roles.RoleDef;
import fr.exemole.bdfserver.api.storage.BalayageStorage;
import fr.exemole.bdfserver.api.storage.BdfExtensionStorage;
import fr.exemole.bdfserver.api.storage.BdfStorageException;
import fr.exemole.bdfserver.api.storage.StorageCheck;
import fr.exemole.bdfserver.api.storage.TableExportStorage;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import fr.exemole.bdfserver.api.subsettree.SubsetTree;
import fr.exemole.bdfserver.conf.ConfConstants;
import fr.exemole.bdfserver.storage.directory.bdfdata.BdfdataDirectories;
import fr.exemole.bdfserver.storage.directory.bdfdata.DirectoryFichotheque;
import fr.exemole.bdfserver.storage.directory.bdfdata.ExceptionHandler;
import fr.exemole.bdfserver.storage.directory.implementations.BalayageStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.BdfExtensionStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.BdfUserStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.ConfigurationStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.GroupStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.PasswordStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.PolicyStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.RedacteurListStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.RoleStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.StorageRootImpl;
import fr.exemole.bdfserver.storage.directory.implementations.ScrutariExportStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.SqlExportStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.TableExportStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.TemplateStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.TreeStorageImpl;
import fr.exemole.bdfserver.storage.directory.implementations.UiStorageImpl;
import fr.exemole.bdfserver.storage.directory.oldversions.DirectoryMigrationEngine;
import fr.exemole.bdfserver.tools.storage.StorageCollection;
import java.io.File;
import java.util.List;
import java.util.Map;
import net.fichotheque.Fichotheque;
import net.fichotheque.FichothequeEditorProvider;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.fiche.ContentChecker;
import net.fichotheque.exportation.scrutari.ScrutariExportDef;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.sphere.Redacteur;
import net.mapeadores.util.html.TrustedHtmlFactory;
import net.mapeadores.util.html.jsoup.CleanTrustedHtmlFactory;
import net.mapeadores.util.html.jsoup.TrustedHtmlFactories;
import net.mapeadores.util.logging.MultiMessageHandler;
import fr.exemole.bdfserver.conf.BdfServerDirs;
import fr.exemole.bdfserver.api.storage.StorageRoot;


/**
 *
 * @author Vincent Calame
 */
public class StorageFactory {

    public final static String SEVERE_FILENAME = "severe.directorystorage.filename";

    private StorageFactory() {
    }

    public static BdfExtensionStorage newBdfExtensionStorage(BdfServerDirs dirs) {
        String subdirName = "extdata";
        return new BdfExtensionStorageImpl(dirs.getSubPath(ConfConstants.VAR_DATA, subdirName), dirs.getSubPath(ConfConstants.VAR_BACKUP, subdirName));
    }

    public static StorageRoot newOutputStorage(BdfServerDirs dirs) {
        return new StorageRootImpl(dirs.getDir(ConfConstants.VAR_OUTPUT));
    }

    public static StorageRoot newCacheStorage(BdfServerDirs dirs) {
        File cacheDir = dirs.getDir(ConfConstants.VAR_CACHE);
        return new StorageRootImpl(new File(cacheDir, "root"));
    }

    public static TableExportStorage newTableExportStorage(BdfServerDirs dirs) {
        String subdirName = "tableexportdata";
        return new TableExportStorageImpl(dirs.getSubPath(ConfConstants.VAR_DATA, subdirName), dirs.getSubPath(ConfConstants.VAR_BACKUP, subdirName));
    }

    public static TemplateStorage newTemplateStorage(BdfServerDirs dirs) {
        String subdirName = "transformationdata";
        return new TemplateStorageImpl(dirs.getSubPath(ConfConstants.VAR_DATA, subdirName), dirs.getSubPath(ConfConstants.VAR_BACKUP, subdirName));
    }

    public static BalayageStorage newBalayageStorage(BdfServerDirs dirs) {
        String subdirName = "balayagedata";
        return new BalayageStorageImpl(dirs.getSubPath(ConfConstants.VAR_DATA, subdirName), dirs.getSubPath(ConfConstants.VAR_BACKUP, subdirName));
    }

    public static void populateWithBdfdata(StorageCollection sc, BdfServerDirs dirs, MultiMessageHandler messageHandler) {
        DirectoryMigrationEngine directoryMigrationEngine = DirectoryMigrationEngine.init(dirs);
        DirectoryStorageCheck check = new DirectoryStorageCheck();
        BdfdataDirectories bdfdataDirectories = BdfdataDirectories.build(dirs);
        InternalExceptionHandler exceptionHandler = new InternalExceptionHandler();
        ContentChecker ficheContentChecker = new ContentChecker(TrustedHtmlFactories.WELLFORMED);
        FichothequeEditorProvider fichothequeEditorProvider = DirectoryFichotheque.buildReadAndWrite(bdfdataDirectories, ficheContentChecker, messageHandler, exceptionHandler);
        if (exceptionHandler.currentBdfStorageException != null) {
            throw exceptionHandler.currentBdfStorageException;
        }
        sc.setFichothequeEditorProvider(fichothequeEditorProvider);
        Fichotheque fichotheque = fichothequeEditorProvider.getFichotheque();
        sc.setPasswordStorage(new PasswordStorageImpl(bdfdataDirectories));

        ConfigurationStorageImpl configurationStorage = new ConfigurationStorageImpl(bdfdataDirectories);
        check.setLangConfiguration(configurationStorage.checkLangConfiguration(messageHandler));
        check.setActiveExtensionList(configurationStorage.checkActiveExtensionList(messageHandler));
        sc.setConfigurationStorage(configurationStorage);

        ScrutariExportStorageImpl scrutariExportStorage = new ScrutariExportStorageImpl(fichotheque, bdfdataDirectories);
        check.setScrutariExportDefList(scrutariExportStorage.check(messageHandler));
        sc.setScrutariExportStorage(scrutariExportStorage);

        SqlExportStorageImpl sqlExportStorage = new SqlExportStorageImpl(fichotheque, bdfdataDirectories);
        check.setSqlExportDefList(sqlExportStorage.check(messageHandler));
        sc.setSqlExportStorage(sqlExportStorage);

        RoleStorageImpl roleStorage = new RoleStorageImpl(bdfdataDirectories);
        check.setRoleDefList(roleStorage.check(messageHandler));
        sc.setRoleStorage(roleStorage);

        GroupStorageImpl groupStorage = new GroupStorageImpl(bdfdataDirectories);
        check.setGroupDefList(groupStorage.check(messageHandler));
        sc.setGroupStorage(groupStorage);

        TreeStorageImpl treeStorageImpl = new TreeStorageImpl(bdfdataDirectories);
        check.setSubsetTreeMap(treeStorageImpl.check(messageHandler));
        sc.setTreeStorage(treeStorageImpl);

        TrustedHtmlFactory uiTrustedHtmlFactory = CleanTrustedHtmlFactory.EXTENDED_WITH_STYLE;
        UiStorageImpl uiStorageImpl = new UiStorageImpl(bdfdataDirectories, uiTrustedHtmlFactory);
        check.setUiCheckMap(uiStorageImpl.check(messageHandler));
        sc.setUiStorage(uiStorageImpl);

        PolicyStorageImpl policyStorageImpl = new PolicyStorageImpl(bdfdataDirectories);
        check.setUserAllow(policyStorageImpl.checkUserAllow(messageHandler));
        check.setSubsetPolicyMap(policyStorageImpl.checkSubsetPolicyMap(messageHandler, fichotheque));
        sc.setPolicyStorage(policyStorageImpl);

        RedacteurListStorageImpl redacteurListStorageImpl = new RedacteurListStorageImpl(bdfdataDirectories);
        check.setRedacteurListMap(redacteurListStorageImpl.checkRedacteurListMap(messageHandler, fichotheque));
        sc.setRedacteurListStorage(redacteurListStorageImpl);

        BdfUserStorageImpl bdfUserStorage = new BdfUserStorageImpl(bdfdataDirectories);
        sc.setBdfUserStorage(bdfUserStorage);

        sc.addMigrationEngine(directoryMigrationEngine);
        sc.setStorageCheck(check);
    }


    private static class InternalExceptionHandler implements ExceptionHandler {

        private BdfStorageException currentBdfStorageException;

        private InternalExceptionHandler() {
        }

        @Override
        public void handleBdfStorageException(BdfStorageException bdfStorageException) {
            currentBdfStorageException = bdfStorageException;
        }

    }


    private static class DirectoryStorageCheck implements StorageCheck {

        private LangConfiguration langConfiguration;
        private List<ScrutariExportDef> scrutariExportDefList;
        private List<SqlExportDef> sqlExportDefList;
        private List<String> activeExtensionList;
        private List<RoleDef> roleDefList;
        private List<GroupDef> groupDefList;
        private Map<Short, SubsetTree> subsetTreeMap;
        private UserAllow userAllow;
        private Map<SubsetKey, Object> subsetPolicyMap;
        private Map<String, List<Redacteur>> redacteurListMap;
        private Map<SubsetKey, UiCheck> uiCheckMap;

        private DirectoryStorageCheck() {
        }

        @Override
        public LangConfiguration getLangConfiguration() {
            return langConfiguration;
        }

        private void setLangConfiguration(LangConfiguration langConfiguration) {
            this.langConfiguration = langConfiguration;
        }

        @Override
        public List<ScrutariExportDef> getScrutariExportDefList() {
            return scrutariExportDefList;
        }

        private void setScrutariExportDefList(List<ScrutariExportDef> scrutariExportDefList) {
            this.scrutariExportDefList = scrutariExportDefList;
        }

        @Override
        public List<SqlExportDef> getSqlExportDefList() {
            return sqlExportDefList;
        }

        private void setSqlExportDefList(List<SqlExportDef> sqlExportDefList) {
            this.sqlExportDefList = sqlExportDefList;
        }

        @Override
        public List<String> getActiveExtensionList() {
            return activeExtensionList;
        }

        private void setActiveExtensionList(List<String> activeExtensionList) {
            this.activeExtensionList = activeExtensionList;
        }

        @Override
        public List<RoleDef> getRoleDefList() {
            return roleDefList;
        }

        private void setRoleDefList(List<RoleDef> roleDefList) {
            this.roleDefList = roleDefList;
        }

        @Override
        public List<GroupDef> getGroupDefList() {
            return groupDefList;
        }

        private void setGroupDefList(List<GroupDef> groupDefList) {
            this.groupDefList = groupDefList;
        }

        @Override
        public Map<Short, SubsetTree> getSubsetTreeMap() {
            return subsetTreeMap;
        }

        private void setSubsetTreeMap(Map<Short, SubsetTree> subsetTreeMap) {
            this.subsetTreeMap = subsetTreeMap;
        }

        @Override
        public UserAllow getUserAllow() {
            return userAllow;
        }

        private void setUserAllow(UserAllow userAllow) {
            this.userAllow = userAllow;
        }

        @Override
        public Map<SubsetKey, Object> getSubsetPolicyMap() {
            return subsetPolicyMap;
        }

        private void setSubsetPolicyMap(Map<SubsetKey, Object> subsetPolicyMap) {
            this.subsetPolicyMap = subsetPolicyMap;
        }

        @Override
        public Map<String, List<Redacteur>> getRedacteurListMap() {
            return redacteurListMap;
        }

        public void setRedacteurListMap(Map<String, List<Redacteur>> redacteurListMap) {
            this.redacteurListMap = redacteurListMap;
        }

        @Override
        public Map<SubsetKey, UiCheck> getUiCheckMap() {
            return uiCheckMap;
        }

        public void setUiCheckMap(Map<SubsetKey, UiCheck> uiCheckMap) {
            this.uiCheckMap = uiCheckMap;
        }

    }


}
