/* BdfServer_HtmlProducers - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.album;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.AlbumDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import java.io.File;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class AlbumHtmlProducerFactory {

    private final static int ALBUM_ADMIN = 1;
    private final static int ALBUM_READ = 2;
    private final static int ILLUSTRATION = 3;

    private AlbumHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        PermissionChecker checker = new PermissionChecker(bdfServer, bdfUser);
        switch (getPageType(page)) {
            case ALBUM_ADMIN: {
                Album album = getAlbum(bdfServer, requestMap, bdfCommandResult);
                checker.checkSubsetAdmin(album);
                switch (page) {
                    case AlbumDomain.ALBUM_ADVANCEDCOMMANDS_PAGE: {
                        return new AlbumAdvancedCommandsHtmlProducer(checker, album);
                    }
                    case AlbumDomain.ALBUM_METADATAFORM_PAGE: {
                        return new AlbumMetadataFormHtmlProducer(checker, album);
                    }
                    default:
                        return null;
                }
            }
            case ALBUM_READ: {
                Album album = getAlbum(bdfServer, requestMap, bdfCommandResult);
                checker.checkSubsetAccess(album);
                switch (page) {
                    case AlbumDomain.ALBUM_PAGE: {
                        return new AlbumHtmlProducer(checker, album);
                    }
                    case AlbumDomain.ILLUSTRATION_UPLOAD_NEW_PAGE: {
                        String appelant = getAppelant(requestMap);
                        return IllustrationUploadHtmlProducer.newNewUpload(bdfServer, bdfUser, album, appelant);
                    }
                    case AlbumDomain.ILLUSTRATION_UPLOAD_REPLACE_PAGE: {
                        String appelant = getAppelant(requestMap);
                        return IllustrationUploadHtmlProducer.newReplaceUpload(bdfServer, bdfUser, album, appelant);
                    }
                    case AlbumDomain.ILLUSTRATION_CHANGE_PAGE: {
                        String appelant = getAppelant(requestMap);
                        if (bdfCommandResult != null) {
                            File tmpFile = (File) bdfCommandResult.getResultObject(BdfInstructionConstants.TMPFILE_OBJ);
                            if (tmpFile != null) {
                                return IllustrationChangeHtmlProducer.newTmpFileCreate(checker, album, appelant, tmpFile.getName());
                            }
                        }
                        String changeType = BdfInstructionUtils.getMandatoryParameter(requestMap, "changetype");
                        if (changeType.equals(AlbumDomain.TMPFILE_CHANGETYPE)) {
                            String tmpFileName = BdfInstructionUtils.getMandatoryParameter(requestMap, "tmpfile");
                            return IllustrationChangeHtmlProducer.newTmpFileChange(checker, album, appelant, tmpFileName);
                        } else if (changeType.equals(AlbumDomain.ILLUSTRATION_CHANGETYPE)) {
                            Illustration illustration = BdfInstructionUtils.getMandatoryIllustration(requestMap, album);
                            String tmpFileName = requestMap.getParameter("tmpfile");
                            return IllustrationChangeHtmlProducer.newIllustrationChange(checker, album, appelant, illustration, tmpFileName);
                        } else {
                            throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue("changetype", changeType));
                        }
                    }
                    default:
                        return null;
                }
            }
            case ILLUSTRATION: {
                Illustration illustration = getIllustration(bdfServer, requestMap, bdfCommandResult);
                switch (page) {
                    case AlbumDomain.ILLUSTRATION_ADMINFORM_PAGE: {
                        checker.checkWrite(illustration);
                        return new IllustrationAdminFormHtmlProducer(checker, illustration);
                    }
                    case AlbumDomain.ILLUSTRATION_ADVANCEDCOMMANDS_PAGE: {
                        checker.checkSubsetAdmin(illustration.getAlbum());
                        return new IllustrationAdvancedCommandsHtmlProducer(checker, illustration);
                    }
                    default:
                        return null;
                }
            }
            default: {
                switch (page) {
                    case AlbumDomain.ALBUM_CREATIONFORM_PAGE: {
                        checker.checkGlobalAdmin();
                        return new AlbumCreationFormHtmlProducer(checker);
                    }
                    case AlbumDomain.ILLUSTRATION_UPLOAD_CONFIRM_PAGE: {
                        String appelant = getAppelant(requestMap);
                        if (bdfCommandResult != null) {
                            File tmpFile = (File) bdfCommandResult.getResultObject(BdfInstructionConstants.TMPFILE_OBJ);
                            if (tmpFile != null) {
                                return new IllustrationUploadConfirmHtmlProducer(checker, appelant, tmpFile.getName());
                            }
                        }
                        return null;
                    }
                    default:
                        return null;
                }
            }
        }
    }

    private static int getPageType(String page) {
        switch (page) {
            case AlbumDomain.ALBUM_ADVANCEDCOMMANDS_PAGE:
            case AlbumDomain.ALBUM_METADATAFORM_PAGE:
                return ALBUM_ADMIN;
            case AlbumDomain.ALBUM_PAGE:
            case AlbumDomain.ILLUSTRATION_UPLOAD_NEW_PAGE:
            case AlbumDomain.ILLUSTRATION_UPLOAD_REPLACE_PAGE:
            case AlbumDomain.ILLUSTRATION_CHANGE_PAGE:
                return ALBUM_READ;
            case AlbumDomain.ILLUSTRATION_ADMINFORM_PAGE:
            case AlbumDomain.ILLUSTRATION_ADVANCEDCOMMANDS_PAGE:
                return ILLUSTRATION;
            default:
                return 0;
        }
    }

    private static Album getAlbum(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Album album = null;
        if (bdfCommandResult != null) {
            album = (Album) bdfCommandResult.getResultObject(BdfInstructionConstants.ALBUM_OBJ);
        }
        if (album == null) {
            album = BdfInstructionUtils.getMandatoryAlbum(bdfServer.getFichotheque(), requestMap);
        }
        return album;
    }

    private static String getAppelant(RequestMap requestMap) {
        return requestMap.getParameter("appelant");
    }

    public static Illustration getIllustration(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Illustration redacteur = null;
        if (bdfCommandResult != null) {
            redacteur = (Illustration) bdfCommandResult.getResultObject(BdfInstructionConstants.ILLUSTRATION_OBJ);
        }
        if (redacteur == null) {
            redacteur = BdfInstructionUtils.getMandatoryIllustration(bdfServer.getFichotheque(), requestMap);
        }
        return redacteur;
    }

}
