/* global Currency,OpenDocument */

/**
 * 
 * @constructor
 * @param {OpenDocument.XmlWriter} xmlWriter
 */
OpenDocument.OdsConverter = function (xmlWriter) {
    this.xmlWriter = xmlWriter;
    this.sheetName = "";
    this.nameMap = {};
    this.flatOds = true;
    this.styleArray = new Array();
    this.rowStyleArray = new Array();
    this.automaticCellStyleArray = new Array();
    this.automaticCellStyleMap = {};
    this.cellStyleNumber = 1;
    this.currencyMap = {};
    this.fixedRows = 0;
    this.fixedColumns = 0;
};

OpenDocument.OdsConverter.convertToBlob = function ($table) {
    var xmlWriter = new OpenDocument.XmlWriter({prettyXml: true});
    var odsConverter = new OpenDocument.OdsConverter(xmlWriter);
    odsConverter.convert($table);
    return new Blob([xmlWriter.xml], {type: OpenDocument.SPREADSHEET_MIMETYPE });
};

OpenDocument.OdsConverter.prototype.convert = function ($table) {
    var converter = this;
    var xw = new OpenDocument.XmlWriter({indentLength: 3});
    var $metadata = $table.children("metadata");
    if ($metadata.length > 0) {
        $metadata = $($metadata[0]);
    }
    var name = this.checkSheetName($metadata.attr("sheet-name"));
    converter.sheetName = name;
    $metadata.children("row").each(function (index, element) {
        converter.rowStyleArray.push(OpenDocument.Style.build(element));
    });
    $metadata.children("cell").each(function (index, element) {
        converter.styleArray.push(OpenDocument.Style.build(element));
    });
    $metadata.children("settings").each(function (index, element) {
        let $element = $(element);
        if ($element.attr("fixed-rows")) {
            converter.fixedRows = $element.attr("fixed-rows");
        }
        if ($element.attr("fixed-columns")) {
            converter.fixedColumns = $element.attr("fixed-columns");
        }
    });
    
    this.columnArray = OpenDocument.OdsConverter.initTableColumns($metadata);
    xw.openTable(name);
    for(let tableColumn of this.columnArray) {
        tableColumn.write(xw);
    }
    var $rows = $table.children("tbody").children("tr");
    $rows.each(function (index, element) {
        var $row = $(element);
        var rowStyleName = $row.attr("class");
        xw.openTableRow(rowStyleName);
        var $cells = $row.children("td");
        $cells.each(function (idx2, el2) {
            _addCell($(el2));
        });
        xw.closeTableRow();
    });
    xw.closeTable(xw);
    this.ods(xw.xml);
    
    
    function _addCell($cell) {
         var cellStyleName = $cell.attr("class");
         var text = $cell.text();
         var colSpan = $cell.attr("colspan");
         if (!text) {
             xw.addEmptyTableCell(cellStyleName);
             return;
         }
         var type = $cell.attr("type");
         switch(type) {
             case "number":
                xw.addNumberTableCell(text, cellStyleName, colSpan);
                break;
             case "date":
                cellStyleName = converter.getCellStyleName("date", cellStyleName);
                xw.addDateTableCell(text, cellStyleName, colSpan);
                break;
            case "currency":
                var currencyCode = $cell.attr("currency");
                cellStyleName = converter.getCellStyleName("currency", cellStyleName, currencyCode);
                xw.addCurrencyTableCell(currencyCode, text, cellStyleName, colSpan);
                break;
             default:
                 xw.addStringTableCell(text, cellStyleName, colSpan);
         }
    }
    
};

OpenDocument.OdsConverter.prototype.getCellStyleName = function (type, parentStyleName, currencyCode) {
    var key = type + ":";
    if (type === "currency") {
        key += currencyCode + ":";
    }
    key += parentStyleName;
    if (this.automaticCellStyleMap.hasOwnProperty(key)) {
        return this.automaticCellStyleMap[key].name;
    }
    var name = OpenDocument.CELLSTYLE_PREFIX + this.cellStyleNumber;

    this.cellStyleNumber++;
    var dataStyleName = null;
    switch(type) {
        case "date":
            dataStyleName = OpenDocument.ISODATE_DATASTYLE_NAME;
            break;
        case "currency":
            var number;
            if (this.currencyMap.hasOwnProperty(currencyCode)) {
                number = this.currencyMap[currencyCode];
            } else {
                number = this.cellStyleNumber;
                this.cellStyleNumber++;
                this.currencyMap[currencyCode] = number;
            }
            dataStyleName = OpenDocument.DATASTYLE_PREFIX + number;
            break;
    }
    var cellStyle = new OpenDocument.CellStyle(name, dataStyleName, parentStyleName);
    this.automaticCellStyleMap[key] = cellStyle;
    this.automaticCellStyleArray.push(cellStyle);
    return name;
};

OpenDocument.OdsConverter.prototype.ods = function (tableXml) {
    var xw = this.xmlWriter;
    xw.appendXMLDeclaration();
    if (this.flatOds) {
        xw.openDocument(OpenDocument.SPREADSHEET_MIMETYPE);
    } else {
        xw.openDocumentContent();
    }
    if ((this.fixedColumns > 0) || (this.fixedRows > 0)) {
        xw
                .openSettings()
                .openConfigItemSet("ooo:view-settings")
                .openConfigItemMapIndexed("Views")
                .openConfigItemMapEntry( "")
                .addConfigItem("ViewId", "string", "view1")
                .openConfigItemMapNamed("Tables")
                .openConfigItemMapEntry(this.sheetName);
        if (this.fixedColumns > 0) {
            xw
                    .addConfigItem("HorizontalSplitMode", "short", "2")
                    .addConfigItem("HorizontalSplitPosition", "int", this.fixedColumns)
                    .addConfigItem("PositionLeft", "int", "0")
                    .addConfigItem("PositionRight", "int", this.fixedColumns);
        }
        if (this.fixedRows > 0) {
            xw
                    .addConfigItem("VerticalSplitMode", "short", "2")
                    .addConfigItem("VerticalSplitPosition", "int", this.fixedRows)
                    .addConfigItem("PositionTop", "int", "0")
                    .addConfigItem("PositionBottom", "int", this.fixedRows);
        }
        xw
                .closeConfigItemMapEntry()
                .closeConfigItemMapNamed()
                .closeConfigItemMapEntry()
                .closeConfigItemMapIndexed()
                .closeConfigItemSet()
                .closeSettings();
    }
    xw.openStyles();
    for(let style of this.styleArray) {
        style.write(xw);
    }
    xw.closeStyles();
    xw.openAutomaticStyles();
     for(let rowStyle of this.rowStyleArray) {
        rowStyle.write(xw);
    }
    for(let column of this.columnArray) {
        column.style.write(xw);
    }
    xw.addDateStyle();
    for(let style of this.automaticCellStyleArray) {
        style.write(xw);
    }
    for(let currencyCode in this.currencyMap) {
        xw.addCurrencyStyle(currencyCode, this.currencyMap[currencyCode]);
    }
    xw
            .closeAutomaticStyles()
            .openBody()
            .openSpreadsheet(xw)
            .write(tableXml)
            .closeSpreadsheet()
            .closeBody();
    if (this.flatOds) {
        xw.closeDocument();
    } else {
        xw.closeDocumentContent();
    }
};

OpenDocument.OdsConverter.prototype.checkSheetName = function (name) {
    if (!name) {
        name = "sheet";
    }
    name = OpenDocument.checkSheetName(name);
    if (!this.nameMap.hasOwnProperty(name)) {
        this.nameMap[name] = true;
        return name;
    } else {
        var p = 2;
        while (true) {
            var newName = name + " (" + p + ")";
            if (!this.nameMap.hasOwnProperty(newName)) {
                this.nameMap[newName] = true;
                return newName;
            }
            p++;
        }
    }

};

OpenDocument.OdsConverter.initTableColumns = function($metadata) {
  var columnArray = new Array();
  $metadata.children("column").each(function (index, element) {
      var number = index + 1;
      var name = OpenDocument.COLUMNSTYLE_PREFIX + number;
      var colspan = 1;
      var colspanAttr = parseInt(element.getAttribute("colspan"));
      if (colspanAttr > 1) {
          colspan = colspanAttr;
      }
      var style = OpenDocument.Style.build(element, name);
      var column = new OpenDocument.TableColumn(name, style, colspan);
      columnArray.push(column);
  });
  return columnArray;
};


