/* FichothequeLib_API - Copyright (c) 2015-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.exportation.transformation;

import java.text.ParseException;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.ValidExtension;


/**
 *
 * @author Vincent Calame
 */
public final class TemplateKey implements Comparable<TemplateKey> {

    public final static String DEFAULT_NAME = "_default";
    private final String key;
    private final TransformationKey transformationKey;
    private final String extension;
    private final String name;
    private final boolean isDefault;

    private TemplateKey(TransformationKey transformationKey, String extension, String name) {
        this.transformationKey = transformationKey;
        this.extension = extension;
        this.name = name;
        if (name == null) {
            this.isDefault = true;
            if (extension == null) {
                this.key = transformationKey.toString() + "/";
            } else {
                this.key = transformationKey.toString() + "/" + "." + extension;
            }
        } else {
            this.isDefault = false;
            if (extension == null) {
                this.key = transformationKey.toString() + "/" + name;
            } else {
                this.key = transformationKey.toString() + "/" + name + "." + extension;
            }
        }
    }

    @Override
    public int hashCode() {
        return key.hashCode();
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        return (this.key.equals(((TemplateKey) other).key));
    }

    @Override
    public String toString() {
        return key;
    }

    @Override
    public int compareTo(TemplateKey otherTemplateKey) {
        return this.key.compareTo(otherTemplateKey.key);
    }

    public boolean isDefault() {
        return isDefault;
    }

    public boolean isSimpleTemplate() {
        return (extension == null);
    }

    public boolean isStreamTemplate() {
        return (extension != null);
    }

    public TransformationKey getTransformationKey() {
        return transformationKey;
    }

    @Nullable
    public String getExtension() {
        return extension;
    }

    public String getName() {
        if (isDefault) {
            return DEFAULT_NAME;
        } else {
            return name;
        }
    }

    public String getCompleteName() {
        if (extension != null) {
            return getName() + "." + extension;
        } else {
            return getName();
        }
    }

    public TemplateKey toDefaultTemplateKey() {
        if (isDefault) {
            return this;
        } else {
            return new TemplateKey(transformationKey, extension, null);
        }
    }

    public static TemplateKey parse(TransformationKey transformationKey, String name) throws ParseException {
        if (transformationKey == null) {
            throw new IllegalArgumentException("transformationKey is null");
        }
        name = checkTemplateName(name);
        return new TemplateKey(transformationKey, null, name);
    }

    public static TemplateKey parse(TransformationKey transformationKey, ValidExtension extension, String name) throws ParseException {
        if (transformationKey == null) {
            throw new IllegalArgumentException("transformationKey is null");
        }
        name = checkTemplateName(name);
        String extensionString = null;
        if (extension != null) {
            extensionString = extension.toString();
        }
        return new TemplateKey(transformationKey, extensionString, name);
    }

    public static TemplateKey parse(String keyString) throws ParseException {
        int idx = keyString.indexOf('/');
        if (idx == -1) {
            throw new ParseException("separator / is missing", 0);
        }
        TransformationKey transformationKey = TransformationKey.parse(keyString.substring(0, idx));
        String name = keyString.substring(idx + 1);
        if (name.length() == 0) {
            return new TemplateKey(transformationKey, null, null);
        }
        int pointIdx = name.indexOf('.');
        if (pointIdx == -1) {
            name = checkTemplateName(name);
            return new TemplateKey(transformationKey, null, name);
        } else if (pointIdx == 0) {
            ValidExtension validExtension = ValidExtension.parse(name.substring(1));
            return new TemplateKey(transformationKey, validExtension.toString(), null);
        } else {
            ValidExtension validExtension = ValidExtension.parse(name.substring(pointIdx + 1));
            name = checkTemplateName(name.substring(0, pointIdx));
            return new TemplateKey(transformationKey, validExtension.toString(), name);
        }
    }

    private static String checkTemplateName(String name) throws ParseException {
        if ((name == null) || (name.isEmpty()) || (name.equals(DEFAULT_NAME))) {
            return null;
        }
        StringUtils.checkTechnicalName(name, true);
        return name;
    }

    public static TemplateKey toDefault(TransformationKey transformationKey) {
        return new TemplateKey(transformationKey, null, null);
    }

    public static TemplateKey toDefault(TransformationKey transformationKey, ValidExtension extension) {
        if (extension == null) {
            return new TemplateKey(transformationKey, null, null);
        } else {
            return new TemplateKey(transformationKey, extension.toString(), null);
        }
    }

}
