/* FichothequeLib_Xml - Copyright (c) 2006-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.extraction;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.run.CorpusExtractor;
import net.fichotheque.extraction.run.ExtraitCorpusResult;
import net.fichotheque.extraction.run.FicheExtractInfo;
import net.fichotheque.extraction.run.MotcleExtractInfo;
import net.fichotheque.namespaces.BdfSpace;
import net.fichotheque.namespaces.BdfXmlSpace;
import net.fichotheque.pointeurs.MotclePointeur;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.fichotheque.utils.pointeurs.PointeurFactory;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.SelectionContextBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class MotcleXMLPart extends XMLPart {

    private final static List<SatelliteInfo> EMPTY_SATELLITELIST = Collections.emptyList();
    private final ExtractionContext extractionContext;
    private final Predicate<FicheMeta> globalFichePredicate;
    private ExtraitCorpusXMLPart extraitCorpusXMLPart = null;

    public MotcleXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
        Predicate<FicheMeta> fichePredicate = extractionContext.getFichePredicate();
        if (fichePredicate == null) {
            fichePredicate = EligibilityUtils.ALL_FICHE_PREDICATE;
        }
        this.globalFichePredicate = fichePredicate;
    }

    public void addMotcle(MotcleExtractInfo motcleExtractInfo) throws IOException {
        Motcle motcle = motcleExtractInfo.getMotcle();
        MotcleFilter motcleFilter = motcleExtractInfo.getMotcleFilter();
        String tagName = motcleFilter.getTagName();
        Croisement croisement = motcleExtractInfo.getCroisement();
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("thesaurus", motcle.getThesaurus().getSubsetKey().getSubsetName());
            addAttribute("id", String.valueOf(motcle.getId()));
            String idalpha = motcle.getIdalpha();
            if (idalpha != null) {
                addAttribute("idalpha", idalpha);
                if (ThesaurusUtils.isBracketsStyle(motcle.getThesaurus().getThesaurusMetadata())) {
                    addAttribute("style", "brackets");
                }
            }
            int poids = ExtractionXMLUtils.getMainPoids(croisement);
            if (poids > 0) {
                addAttribute("poids", String.valueOf(poids));
            }
            Attribute urlAttribute = motcle.getAttributes().getAttribute(BdfSpace.URL_KEY);
            if (urlAttribute != null) {
                addAttribute("url", urlAttribute.getFirstValue());
            }
            List<SatelliteInfo> satelliteList = getSatelliteList(motcle);
            if ((motcleFilter.isNoneFiltering()) && (satelliteList.isEmpty())) {
                closeEmptyTag();
                return;
            }
            endOpenTag();
            addSatelliteList(satelliteList);
            ExtractionXMLUtils.writeCroisement(this, croisement);
        } else if (motcleFilter.isNoneFiltering()) {
            return;
        }
        if (motcleFilter.withLevel()) {
            addSimpleElement("level", String.valueOf(motcle.getLevel()));
        }
        if (motcleFilter.withLabels()) {
            LabelUtils.addLabels(this, motcle.getLabels());
        }
        if (motcleFilter.withIntitule()) {
            ExtractionXMLUtils.writeIntitule(this, motcle, "fichestyle", extractionContext);
        }
        MotcleFilter childrenFilter = motcleFilter.getChildrenFilter();
        if (childrenFilter != null) {
            for (Motcle child : motcle.getChildList()) {
                MotcleExtractInfo childExtractInfo = ExtractionXMLUtils.toMotcleExtractInfo(child, childrenFilter);
                addMotcle(childExtractInfo);
            }
        }
        MotcleFilter parentFilter = motcleFilter.getParentFilter();
        if (parentFilter != null) {
            Motcle parentMotcle = motcle.getParent();
            addLinkedMotcle(parentMotcle, motcleFilter.getParentFilter());
        }
        MotcleFilter previousFilter = motcleFilter.getPreviousFilter();
        if (previousFilter != null) {
            Motcle previousMotcle = ThesaurusUtils.getPrevious(motcle);
            addLinkedMotcle(previousMotcle, previousFilter);
        }
        MotcleFilter nextFilter = motcleFilter.getNextFilter();
        if (nextFilter != null) {
            Motcle nextMotcle = ThesaurusUtils.getNext(motcle);
            addLinkedMotcle(nextMotcle, nextFilter);
        }
        addFicheRattachement(motcle, motcleFilter.getFicheRattachementDefList());
        addIndexation(motcle, motcleFilter);
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void addLinkedMotcle(Motcle linkedMotcle, MotcleFilter linkedMotcleFilter) throws IOException {
        if (linkedMotcle == null) {
            return;
        }
        addMotcle(ExtractionXMLUtils.toMotcleExtractInfo(linkedMotcle, linkedMotcleFilter));
    }

    private void addFicheRattachement(Motcle motcle, List<FicheRattachementDef> ficheRattachementDefList) throws IOException {
        if (ficheRattachementDefList.isEmpty()) {
            return;
        }
        int motcleid = motcle.getId();
        FicheXMLPart ficheXMLPart = new FicheXMLPart(getXMLWriter(), extractionContext);
        for (FicheRattachementDef ficheRattachementDef : ficheRattachementDefList) {
            List<Corpus> corpusList = toCorpusList(motcle.getThesaurus(), ficheRattachementDef);
            for (Corpus corpus : corpusList) {
                FicheMeta ficheMeta = corpus.getFicheMetaById(motcleid);
                if ((ficheMeta != null) && (globalFichePredicate.test(ficheMeta))) {
                    FicheExtractInfo ficheExtractInfo = ExtractionXMLUtils.toFicheExtractInfo(ficheMeta, ficheRattachementDef.getFicheFilter());
                    ficheXMLPart.addFiche(ficheExtractInfo);
                }
            }
        }
    }

    private void addIndexation(Motcle motcle, MotcleFilter motcleFilter) throws IOException {
        MotclePointeur motclePointeur = PointeurFactory.newMotclePointeur(motcle.getThesaurus());
        motclePointeur.setCurrentSubsetItem(motcle);
        List<ExtraitCorpusDef> extraitCorpusDefList = motcleFilter.getIndexationExtraitCorpusDefList();
        if (extraitCorpusDefList.isEmpty()) {
            return;
        }
        Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
        SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                .setFichePredicate(extractionContext.getFichePredicate())
                .toSelectionContext();
        for (ExtraitCorpusDef extraitCorpusDef : extraitCorpusDefList) {
            CorpusExtractor corpusExtractor = extractionContext.getExtractorProvider().getCorpusExtractor(extraitCorpusDef);
            FicheSelector indexationFicheSelector = FicheSelectorBuilder.init(selectionContext).addAll(extraitCorpusDef.getFicheQueryList()).toFicheSelector();
            for (Corpus corpus : indexationFicheSelector.getCorpusList()) {
                Croisements croisements;
                if (extraitCorpusDef.isDescendantAxis()) {
                    croisements = motclePointeur.getDescendantAxisCroisementList(corpus);
                } else {
                    croisements = motclePointeur.getCroisements(corpus);
                }
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    FicheMeta ficheMeta = (FicheMeta) entry.getSubsetItem();
                    Croisement filteredCroisement = indexationFicheSelector.isSelected(ficheMeta, entry.getCroisement());
                    if (filteredCroisement != null) {
                        corpusExtractor.add(ficheMeta, filteredCroisement);
                    }
                }
            }
            if (!corpusExtractor.containsFiche()) {
                continue;
            }
            if (extraitCorpusXMLPart == null) {
                extraitCorpusXMLPart = new ExtraitCorpusXMLPart(getXMLWriter(), extractionContext);
            }
            for (ExtraitCorpusResult extraitCorpusResult : corpusExtractor.getExtraitCorpusResultList()) {
                extraitCorpusXMLPart.addExtraitcorpus(extraitCorpusResult);
            }
        }
    }

    private List<Corpus> toCorpusList(Thesaurus thesaurus, FicheRattachementDef ficheRattachementDef) {
        List<SubsetKey> subsetKeyList = ficheRattachementDef.getRattachementSubsetKeyList();
        if (subsetKeyList.isEmpty()) {
            return thesaurus.getSatelliteCorpusList();
        }
        Fichotheque fichotheque = thesaurus.getFichotheque();
        List<Corpus> list = new ArrayList<Corpus>();
        for (SubsetKey subsetKey : subsetKeyList) {
            Corpus corpus = (Corpus) fichotheque.getSubset(subsetKey);
            if (corpus != null) {
                Subset masterSubset = corpus.getMasterSubset();
                if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
                    list.add(corpus);
                }
            }
        }
        return list;
    }

    private void addSatelliteList(List<SatelliteInfo> satelliteList) throws IOException {
        for (SatelliteInfo satelliteInfo : satelliteList) {
            String lang = satelliteInfo.lang;
            startOpenTag("satellite");
            addAttribute("corpus", satelliteInfo.corpusName);
            if (lang != null) {
                addAttribute("lang", lang);
            }
            closeEmptyTag();
        }
    }

    private List<SatelliteInfo> getSatelliteList(Motcle motcle) {
        Thesaurus thesaurus = motcle.getThesaurus();
        List<Corpus> satelliteCorpusList = thesaurus.getSatelliteCorpusList();
        if (satelliteCorpusList.isEmpty()) {
            return EMPTY_SATELLITELIST;
        }
        HashSet<String> existingString = new HashSet<String>();
        for (Corpus satelliteCorpus : satelliteCorpusList) {
            FicheMeta ficheMeta = satelliteCorpus.getFicheMetaById(motcle.getId());
            if ((ficheMeta != null) && (globalFichePredicate.test(ficheMeta))) {
                existingString.add(satelliteCorpus.getSubsetKey().getSubsetName());
            }
        }
        if (existingString.isEmpty()) {
            return EMPTY_SATELLITELIST;
        }
        Attribute satelliteAttribute = motcle.getAttributes().getAttribute(BdfXmlSpace.SATELLITE_ATTRIBUTEKEY);
        if (satelliteAttribute != null) {
            if (satelliteAttribute.getFirstValue().startsWith("_")) {
                return EMPTY_SATELLITELIST;
            } else {
                for (String value : satelliteAttribute) {
                    if (existingString.contains(value)) {
                        return Collections.singletonList(new SatelliteInfo(value, null));
                    }
                }
                return EMPTY_SATELLITELIST;
            }
        }
        List<SatelliteInfo> resultList = new ArrayList<SatelliteInfo>();
        satelliteAttribute = thesaurus.getThesaurusMetadata().getAttributes().getAttribute(BdfXmlSpace.SATELLITE_ATTRIBUTEKEY);
        if (satelliteAttribute != null) {
            for (String value : satelliteAttribute) {
                if (existingString.contains(value)) {
                    resultList.add(new SatelliteInfo(value, null));
                }
            }
        }
        for (Attribute attribute : thesaurus.getThesaurusMetadata().getAttributes()) {
            String attributeKey = attribute.getAttributeKey().toString();
            if (attributeKey.startsWith(BdfXmlSpace.SATELLITE_ATTRIBUTEKEY_PREFIX)) {
                String lang = attributeKey.substring(BdfXmlSpace.SATELLITE_ATTRIBUTEKEY_PREFIX.length());
                if (!lang.isEmpty()) {
                    for (String value : attribute) {
                        if (existingString.contains(value)) {
                            resultList.add(new SatelliteInfo(value, lang));
                        }
                    }
                }
            }
        }
        return resultList;
    }


    private static class SatelliteInfo {

        private final String corpusName;
        private final String lang;

        private SatelliteInfo(String corpusName, String lang) {
            this.corpusName = corpusName;
            this.lang = lang;
        }


    }

}
